@@ -94,7 +94,10 @@ class SBML:
         if errors is not None:
             critical_errors = errors[errors[SBML_DEFS.ERROR_SEVERITY] >= 2]
             critical_errors = set(critical_errors[SBML_DEFS.ERROR_DESCRIPTION].unique())
-            known_errors = {"<layout> must have 'id' and may have 'name'"}
+            known_errors = {
+                "<layout> must have 'id' and may have 'name'",
+                "Missing value for the 'compartment' attribute",
+            }
 
             found_known_errors = known_errors.intersection(critical_errors)
             if len(found_known_errors) > 0:
@@ -257,6 +260,12 @@ class SBML:
                     }
                 )
 
+        if len(compartments) == 0:
+            logger.warning(
+                "No compartments were found in the SBML model. using default compartment GO CELLULAR_COMPONENT"
+            )
+            return sbml_dfs_utils.stub_compartments(with_source=True)
+
         return pd.DataFrame(compartments).set_index(SBML_DFS.C_ID)
 
     def _define_cspecies(self) -> pd.DataFrame:
@@ -295,6 +304,16 @@ class SBML:
             comp_species_df[SBML_DFS.SC_NAME], "SC"
         )
 
+        # replace empty strings for c_id with None so the resolve function will properly fill these
+        comp_species_df[SBML_DFS.C_ID] = comp_species_df[SBML_DFS.C_ID].replace(
+            "", None
+        )
+
+        # if all entries are missing then c_id was likely stubbed with sbml_dfs_utils.stub_compartments()
+        if all(comp_species_df[SBML_DFS.C_ID].isnull()):
+            default_cid = sbml_dfs_utils.stub_compartments().index[0]
+            comp_species_df[SBML_DFS.C_ID] = default_cid
+
         return comp_species_df
 
     def _define_fbc_gene_products(self) -> list[dict]:
@@ -424,6 +443,9 @@ class SBML:
             columns={"new_id": SBML_DFS.S_ID}
         )[CSPECIES_VARS]
 
+        # Internal validation: Check consistency between species and compartmentalized species
+        _validate_species_consistency(species, compartmentalized_species)
+
         return species, compartmentalized_species
 
 
@@ -607,12 +629,113 @@ def sbml_dfs_from_sbml(self, sbml_model: SBML, compartment_aliases: dict | None
     # 2. Process species and compartmentalized species
     self.species, self.compartmentalized_species = sbml_model._define_species()
 
+    # Refine compartments to only those actually used by species
+    self.compartments = _refine_compartments(
+        self.compartments, self.compartmentalized_species
+    )
+
     # 3. Process reactions and their participating species
     self.reactions, self.reaction_species = sbml_model._define_reactions()
 
     return self
 
 
+def _validate_species_consistency(species_df, compartmentalized_species_df):
+    """Validate consistency between species and compartmentalized species tables.
+
+    Parameters
+    ----------
+    species_df : pd.DataFrame
+        DataFrame of species with s_id as index
+    compartmentalized_species_df : pd.DataFrame
+        DataFrame of compartmentalized species with s_id column
+
+    Raises
+    ------
+    ValueError
+        If there are inconsistencies between the two tables
+    """
+    species_ids_in_species = set(species_df.index)
+    species_ids_in_cspecies = set(compartmentalized_species_df[SBML_DFS.S_ID])
+
+    missing_in_species = species_ids_in_cspecies - species_ids_in_species
+    missing_in_cspecies = species_ids_in_species - species_ids_in_cspecies
+
+    if missing_in_species:
+        raise ValueError(
+            f"Species extraction validation error: {len(missing_in_species)} species IDs "
+            f"found in compartmentalized_species but missing from species table. "
+            f"Missing species: {sorted(list(missing_in_species))[:10]}"
+        )
+
+    if missing_in_cspecies:
+        raise ValueError(
+            f"Species extraction validation error: {len(missing_in_cspecies)} species IDs "
+            f"found in species table but missing from compartmentalized_species table. "
+            f"Missing from compartmentalized_species: {sorted(list(missing_in_cspecies))[:10]}"
+        )
+
+
+def _refine_compartments(compartments_df, compartmentalized_species_df):
+    """Refine compartments to only those actually used by compartmentalized species.
+
+    This function filters the compartments DataFrame to include only compartments
+    that are referenced by compartmentalized species, and validates that all
+    required compartments exist.
+
+    Parameters
+    ----------
+    compartments_df : pd.DataFrame
+        DataFrame of all extracted compartments with c_id as index
+    compartmentalized_species_df : pd.DataFrame
+        DataFrame of compartmentalized species with c_id column
+
+    Returns
+    -------
+    pd.DataFrame
+        Filtered compartments DataFrame containing only used compartments
+
+    Raises
+    ------
+    ValueError
+        If compartmentalized species reference compartments that don't exist
+    """
+    compartment_ids_in_compartments = set(compartments_df.index)
+    compartment_ids_in_cspecies = set(compartmentalized_species_df[SBML_DFS.C_ID])
+    # ignore None in compartment_ids_in_cspecies - this can likely be resolved downstream
+    compartment_ids_in_cspecies = {
+        c for c in compartment_ids_in_cspecies if c is not None
+    }
+
+    # Check that all required compartments exist
+    missing_compartments = compartment_ids_in_cspecies - compartment_ids_in_compartments
+    if missing_compartments:
+        raise ValueError(
+            f"Compartment refinement error: {len(missing_compartments)} compartment IDs "
+            f"found in compartmentalized_species but missing from compartments table. "
+            f"Missing compartments: {sorted(list(missing_compartments))[:10]}"
+        )
+
+    # Filter to only compartments that are actually used
+    unused_compartments = compartment_ids_in_compartments - compartment_ids_in_cspecies
+    if unused_compartments:
+        logger.debug(
+            f"Filtering out {len(unused_compartments)} unused compartments from compartments table. "
+            f"Unused compartments: {sorted(list(unused_compartments))[:10]}"
+        )
+
+    # Return only the compartments that are referenced by species
+    used_compartments = compartments_df.loc[
+        compartments_df.index.isin(compartment_ids_in_cspecies)
+    ]
+
+    logger.debug(
+        f"Refined compartments: {len(used_compartments)} of {len(compartments_df)} compartments retained"
+    )
+
+    return used_compartments
+
+
 def _define_compartments_missing_cvterms(
     comp: libsbml.Compartment, aliases: dict
 ) -> dict[str, Any]:
