@@ -1,24 +1,24 @@
 from __future__ import annotations
 
 import os
+import warnings
 from typing import Any
 
-from fs import open_fs
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", message="pkg_resources is deprecated")
+    from fs import open_fs
 import numpy as np
 import pandas as pd
 
-from napistu import identifiers
-from napistu import sbml_dfs_core
-from napistu import sbml_dfs_utils
-from napistu import source
-
-from napistu.constants import BQB
-from napistu.constants import SBML_DFS
-from napistu.constants import SBML_DFS_SCHEMA
-from napistu.constants import IDENTIFIERS
-from napistu.constants import MINI_SBO_FROM_NAME
-from napistu.constants import SBOTERM_NAMES
-
+from napistu import identifiers, sbml_dfs_core, sbml_dfs_utils, source
+from napistu.constants import (
+    BQB,
+    IDENTIFIERS,
+    MINI_SBO_FROM_NAME,
+    SBML_DFS,
+    SBML_DFS_SCHEMA,
+    SBOTERM_NAMES,
+)
 from napistu.modify.constants import VALID_ANNOTATION_TYPES
 
 
@@ -512,11 +512,7 @@ def format_curated_entities(
 
             is_identified = not new_entity_series.isna()["uri"]
             if is_identified:
-                id = [
-                    identifiers.format_uri(
-                        new_entity_series["uri"], biological_qualifier_type=BQB.IS
-                    )
-                ]
+                id = [identifiers.format_uri(new_entity_series["uri"], bqb=BQB.IS)]
             else:
                 id = [
                     {
