@@ -2,15 +2,19 @@ from __future__ import annotations
 
 import copy
 import logging
-from typing import Any, Optional, Union
+from typing import TYPE_CHECKING, Any, Mapping, MutableMapping, Optional, Union
 
 import igraph as ig
 import pandas as pd
 
+if TYPE_CHECKING:
+    pass
+
 from napistu import utils
 from napistu.constants import (
     ENTITIES_TO_ENTITY_DATA,
     ENTITIES_W_DATA,
+    MINI_SBO_TO_NAME,
     SBML_DFS,
 )
 from napistu.ingestion.constants import DEFAULT_PRIORITIZED_PATHWAYS
@@ -21,6 +25,7 @@ from napistu.network.constants import (
     EDGE_DIRECTION_MAPPING,
     EDGE_REVERSAL_ATTRIBUTE_MAPPING,
     ENTITIES_TO_ATTRS,
+    IGRAPH_DEFS,
     NAPISTU_GRAPH,
     NAPISTU_GRAPH_EDGES,
     NAPISTU_GRAPH_NODE_TYPES,
@@ -69,7 +74,7 @@ class NapistuGraph(ig.Graph):
         Add edge data from SBML_dfs to the graph.
     add_topology_weights(base_score=2, protein_multiplier=1, metabolite_multiplier=3, unknown_multiplier=10, scale_multiplier_by_meandegree=True, inplace=True)
         Add topology-based weights to graph edges.
-    add_sbml_dfs_summaries(sbml_dfs, summary_types=None, priority_pathways=None, stratify_by_bqb=True, characteristic_only=False, dogmatic=False, inplace=True)
+    add_sbml_dfs_summaries(sbml_dfs, summary_types=VALID_VERTEX_SBML_DFS_SUMMARIES, priority_pathways=DEFAULT_PRIORITIZED_PATHWAYS, stratify_by_bqb=True, characteristic_only=False, dogmatic=False, inplace=True)
         Add vertex summaries from SBML_dfs to the graph vertices.
     add_vertex_data(sbml_dfs, side_loaded_attributes, mode='fresh', overwrite=False, inplace=True)
         Add vertex data from SBML_dfs to the graph.
@@ -81,10 +86,16 @@ class NapistuGraph(ig.Graph):
         Load a NapistuGraph from a pickle file.
     get_edge_dataframe()
         Return graph edges as a pandas DataFrame.
+    get_edge_series(attribute_name)
+        Return a single edge attribute as a pandas Series.
     get_metadata(key=None)
         Get metadata from the graph.
     get_vertex_dataframe()
         Return graph vertices as a pandas DataFrame.
+    get_vertex_series(attribute_name)
+        Return a single vertex attribute as a pandas Series.
+    remove_attributes(attribute_type, attributes)
+        Remove specified attributes from vertices or edges.
     remove_isolated_vertices(node_types='reactions')
         Remove isolated vertices from the graph.
     reverse_edges()
@@ -204,7 +215,7 @@ class NapistuGraph(ig.Graph):
         entity_type: str,
         target_entity: Optional[str] = None,
         table_names: Optional[list[str]] = None,
-        name_prefix: bool = True,
+        add_name_prefixes: bool = True,
         mode: str = ADDING_ENTITY_DATA_DEFS.FRESH,
         overwrite: bool = False,
         inplace: bool = True,
@@ -230,7 +241,7 @@ class NapistuGraph(ig.Graph):
             or species data on edges (if such use case exists).
         table_names : Optional[list[str]], default=None
             Specific table names to include. If None, includes all available tables.
-        name_prefix : bool, default=True
+        add_name_prefixes : bool, default=True
             Whether to prefix attribute names with table name (e.g., "table_name_column_name")
         mode : str, default="fresh"
             Either "fresh" (replace existing) or "extend" (add new attributes only)
@@ -311,7 +322,7 @@ class NapistuGraph(ig.Graph):
         entity_attrs = ng_utils.create_entity_attrs_from_data_tables(
             entity_data_dict=entity_data_dict,
             table_names=table_names,
-            name_prefix=name_prefix,
+            add_name_prefixes=add_name_prefixes,
         )
 
         if not entity_attrs:
@@ -551,11 +562,12 @@ class NapistuGraph(ig.Graph):
     def add_sbml_dfs_summaries(
         self,
         sbml_dfs: SBML_dfs,
-        summary_types=VALID_VERTEX_SBML_DFS_SUMMARIES,
-        priority_pathways=DEFAULT_PRIORITIZED_PATHWAYS,
-        stratify_by_bqb=True,
-        characteristic_only=False,
-        dogmatic=False,
+        summary_types: list[str] = VALID_VERTEX_SBML_DFS_SUMMARIES,
+        priority_pathways: list[str] = DEFAULT_PRIORITIZED_PATHWAYS,
+        stratify_by_bqb: bool = True,
+        characteristic_only: bool = False,
+        dogmatic: bool = False,
+        add_name_prefixes: bool = False,
         mode: str = ADDING_ENTITY_DATA_DEFS.FRESH,
         overwrite: bool = False,
         inplace: bool = True,
@@ -580,6 +592,9 @@ class NapistuGraph(ig.Graph):
             Whether to include only characteristic identifiers. Default is False.
         dogmatic : bool, optional
             Whether to use dogmatic mode. Default is False.
+        add_name_prefixes : bool, default False
+            If True, add prefixes to column names: 'source_' for source data
+            and 'ontology_' for ontology data
         mode : str
             Either "fresh" (replace existing) or "extend" (add new attributes only)
         overwrite : bool
@@ -608,6 +623,7 @@ class NapistuGraph(ig.Graph):
             stratify_by_bqb=stratify_by_bqb,
             characteristic_only=characteristic_only,
             dogmatic=dogmatic,
+            add_name_prefixes=add_name_prefixes,
         )
 
         logger.info("Creating graph attributes")
@@ -886,6 +902,38 @@ class NapistuGraph(ig.Graph):
         """
         return super().get_edge_dataframe()
 
+    def get_edge_series(self, attribute_name: str) -> pd.Series:
+        """
+        Get a single edge attribute as a pandas Series.
+
+        Parameters
+        ----------
+        attribute_name : str
+            Name of the edge attribute to extract
+
+        Returns
+        -------
+        pd.Series
+            Series with MultiIndex (from, to) as index and attribute values as data
+
+        Raises
+        ------
+        KeyError
+            If the attribute does not exist on any edges
+        """
+        if attribute_name not in self.es.attributes():
+            raise KeyError(f"Edge attribute '{attribute_name}' not found")
+
+        edge_tuples = [
+            (e[NAPISTU_GRAPH_EDGES.FROM], e[NAPISTU_GRAPH_EDGES.TO]) for e in self.es
+        ]
+        attribute_values = [e[attribute_name] for e in self.es]
+
+        multi_index = pd.MultiIndex.from_tuples(
+            edge_tuples, names=[NAPISTU_GRAPH_EDGES.FROM, NAPISTU_GRAPH_EDGES.TO]
+        )
+        return pd.Series(attribute_values, index=multi_index, name=attribute_name)
+
     def get_metadata(self, key: Optional[str] = None) -> Any:
         """
         Get metadata from the graph.
@@ -904,6 +952,45 @@ class NapistuGraph(ig.Graph):
             return self._metadata.copy()
         return self._metadata.get(key)
 
+    def get_summary(self) -> Mapping[str, Any]:
+        """
+        Get summary statistics for the graph.
+
+        Returns
+        -------
+        Mapping[str, Any]
+            A dictionary of summary statistics.
+            - n_vertices: Number of vertices
+            - vertex_node_type_dict: Dictionary of node type counts
+            - vertex_species_type_dict: Dictionary of species type counts
+            - vertex_attributes: List of vertex attributes
+            - n_edges: Number of edges
+            - sbo_name_counts_dict: Dictionary of SBO name counts
+            - edge_attributes: List of edge attributes
+        """
+
+        stats: MutableMapping[str, Any] = {}
+
+        # vertex summaries
+        vertex_df = self.get_vertex_dataframe()
+        stats["n_vertices"] = vertex_df.shape[0]
+        stats["vertex_node_type_dict"] = vertex_df.value_counts("node_type").to_dict()
+        stats["vertex_species_type_dict"] = vertex_df.value_counts(
+            "species_type"
+        ).to_dict()
+        stats["vertex_attributes"] = vertex_df.columns.tolist()
+
+        # edge summaries
+        stats["n_edges"] = len(self.es)
+        sbo_name_counts = self.get_edge_series(
+            NAPISTU_GRAPH_EDGES.SBO_TERM
+        ).value_counts()
+        sbo_name_counts.index = sbo_name_counts.index.map(MINI_SBO_TO_NAME)
+        stats["sbo_name_counts_dict"] = sbo_name_counts.to_dict()
+        stats["edge_attributes"] = self.es.attributes()
+
+        return stats
+
     def get_vertex_dataframe(self) -> pd.DataFrame:
         """
         Get vertices as a Pandas DataFrame.
@@ -914,7 +1001,38 @@ class NapistuGraph(ig.Graph):
         pandas.DataFrame
             A table with one row per vertex.
         """
-        return super().get_vertex_dataframe()
+        df = super().get_vertex_dataframe()
+        df.index = df.index.rename(IGRAPH_DEFS.INDEX)
+        return df
+
+    def get_vertex_series(self, attribute_name: str) -> pd.Series:
+        """
+        Get a single vertex attribute as a pandas Series.
+
+        Parameters
+        ----------
+        attribute_name : str
+            Name of the vertex attribute to extract
+
+        Returns
+        -------
+        pd.Series
+            Series with vertex names as index and attribute values as data
+
+        Raises
+        ------
+        KeyError
+            If the attribute does not exist on any vertices
+        """
+        if attribute_name not in self.vs.attributes():
+            raise KeyError(f"Vertex attribute '{attribute_name}' not found")
+
+        vertex_names = [v[NAPISTU_GRAPH_VERTICES.NAME] for v in self.vs]
+        attribute_values = [v[attribute_name] for v in self.vs]
+
+        series = pd.Series(attribute_values, index=vertex_names, name=attribute_name)
+        series.index.name = NAPISTU_GRAPH_VERTICES.NAME
+        return series
 
     def set_graph_attrs(
         self,
@@ -981,6 +1099,82 @@ class NapistuGraph(ig.Graph):
             )
             self.set_metadata(reaction_attrs=merged_reactions)
 
+    def show_summary(self) -> None:
+        """
+        Show summary statistics for the graph.
+        """
+        summary_stats = self.get_summary()
+        summary_table = ng_utils.format_napistu_graph_summary(summary_stats)
+        utils.show(summary_table)
+
+    def remove_attributes(self, attribute_type: str, attributes: list[str]) -> None:
+        """
+        Remove specified attributes from vertices or edges.
+
+        This method removes the specified attributes from either vertices or edges
+        and warns if any of the attributes to be removed are not associated with
+        any vertices/edges in the graph.
+
+        Parameters
+        ----------
+        attribute_type : str
+            Type of attributes to remove. Must be either NAPISTU_GRAPH.VERTICES or NAPISTU_GRAPH.EDGES.
+        attributes : list[str]
+            List of attribute names to remove from the specified attribute type.
+
+        Returns
+        -------
+        None
+            The graph is modified in-place.
+
+        Raises
+        ------
+        ValueError
+            If attribute_type is not NAPISTU_GRAPH.VERTICES or NAPISTU_GRAPH.EDGES.
+
+        Examples
+        --------
+        Remove vertex attributes:
+        >>> graph.remove_attributes(NAPISTU_GRAPH.VERTICES, ["species_type", "node_type"])
+
+        Remove edge attributes:
+        >>> graph.remove_attributes(NAPISTU_GRAPH.EDGES, ["weight", "stoichiometry"])
+
+        Notes
+        -----
+        This method will warn about attributes that don't exist in the graph
+        but will continue to remove the attributes that do exist.
+        """
+
+        # Get the appropriate attribute container
+        if attribute_type == NAPISTU_GRAPH.VERTICES:
+            attr_container = self.vs
+        elif attribute_type == NAPISTU_GRAPH.EDGES:
+            attr_container = self.es
+        else:
+            raise ValueError(
+                f"attribute_type must be one of: {NAPISTU_GRAPH.VERTICES}, {NAPISTU_GRAPH.EDGES}, got '{attribute_type}'"
+            )
+
+        missing_attrs = set(attributes) - set(attr_container.attributes())
+        if len(missing_attrs) > 0:
+            logger.warning(
+                f"The following {attribute_type} attributes are not associated with any {attribute_type} and will be skipped: {sorted(missing_attrs)}"
+            )
+
+        existing_attrs_to_remove = set(attributes) & set(attr_container.attributes())
+        # Remove existing attributes
+        if existing_attrs_to_remove:
+            for attr_name in existing_attrs_to_remove:
+                del attr_container[attr_name]
+
+            # Clean up metadata for removed attributes
+            self._cleanup_removed_attributes_metadata(
+                attribute_type, existing_attrs_to_remove
+            )
+
+        return None
+
     def remove_isolated_vertices(self, node_types: str = SBML_DFS.REACTIONS):
         """
         Remove vertices that have no edges (degree 0) from the graph.
@@ -1245,8 +1439,8 @@ class NapistuGraph(ig.Graph):
             Dictionary mapping transformation names to functions
         """
         self._transform_entity_attributes(
-            entity_type="reactions",
-            target_entity="edges",
+            entity_type=SBML_DFS.REACTIONS,
+            target_entity=NAPISTU_GRAPH.EDGES,
             keep_raw_attributes=keep_raw_attributes,
             custom_transformations=custom_transformations,
         )
@@ -1267,8 +1461,8 @@ class NapistuGraph(ig.Graph):
             Dictionary mapping transformation names to functions
         """
         self._transform_entity_attributes(
-            entity_type="species",
-            target_entity="vertices",
+            entity_type=SBML_DFS.SPECIES,
+            target_entity=NAPISTU_GRAPH.VERTICES,
             keep_raw_attributes=keep_raw_attributes,
             custom_transformations=custom_transformations,
         )
@@ -1405,6 +1599,50 @@ class NapistuGraph(ig.Graph):
                     current_upstream * multiplier
                 )
 
+    def _cleanup_removed_attributes_metadata(
+        self, attribute_type: str, removed_attributes: set[str]
+    ) -> None:
+        """
+        Clean up metadata for removed attributes.
+
+        This method removes entries from the metadata that are no longer relevant
+        after attributes have been removed from the graph.
+
+        Parameters
+        ----------
+        attribute_type : str
+            Type of attributes that were removed (NAPISTU_GRAPH.VERTICES or NAPISTU_GRAPH.EDGES)
+        removed_attributes : set[str]
+            Set of attribute names that were removed
+        """
+        # Map attribute_type to entity_type for metadata
+        if attribute_type == NAPISTU_GRAPH.VERTICES:
+            entity_type = SBML_DFS.SPECIES
+        else:  # edges
+            entity_type = SBML_DFS.REACTIONS
+
+        # Clean up raw attributes metadata
+        if NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES in self._metadata:
+            raw_attrs = self._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES]
+            if entity_type in raw_attrs:
+                for attr_name in removed_attributes:
+                    raw_attrs[entity_type].pop(attr_name, None)
+                # Remove empty entity type if no raw attributes left
+                if not raw_attrs[entity_type]:
+                    del raw_attrs[entity_type]
+
+        # Clean up transformations applied metadata
+        if NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED in self._metadata:
+            transformations = self._metadata[
+                NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED
+            ]
+            if entity_type in transformations:
+                for attr_name in removed_attributes:
+                    transformations[entity_type].pop(attr_name, None)
+                # Remove empty entity type if no transformations left
+                if not transformations[entity_type]:
+                    del transformations[entity_type]
+
     def _compare_and_merge_attrs(
         self,
         new_attrs: dict,
@@ -1790,6 +2028,12 @@ class NapistuGraph(ig.Graph):
             entity_data, left_on=merge_keys, right_index=True, how="left"
         )
 
+        # validate that there wasn't a 1-to-many merge
+        if graph_with_attrs.shape[0] != graph_df.shape[0]:
+            raise ValueError(
+                f"1-to-many merge occurred when adding {entity_data.shape[0]} {entity_data.index.names} to graph {target_entity}"
+            )
+
         # Add new attributes directly to the graph
         added_count = 0
         for attr_name in attrs_to_add:
@@ -1805,6 +2049,8 @@ class NapistuGraph(ig.Graph):
             f"Added {added_count} {entity_name} attributes to graph: {attrs_to_add}"
         )
 
+        return None
+
     def _transform_entity_attributes(
         self,
         entity_type: str,
@@ -1979,6 +2225,8 @@ class NapistuGraph(ig.Graph):
             f"Transformed {len(attrs_to_transform)} {target_entity} attributes: {list(attrs_to_transform)}"
         )
 
+        return None
+
 
 def _apply_edge_reversal_mapping(edges_df: pd.DataFrame) -> pd.DataFrame:
     """
