@@ -7,20 +7,23 @@ import pandas as pd
 import pytest
 from fs.errors import ResourceNotFound
 
-from napistu.constants import SBML_DFS
+from napistu.constants import SBML_DFS, SBOTERM_NAMES
 from napistu.network import ng_utils
 from napistu.network.constants import (
     DEFAULT_WT_TRANS,
     GRAPH_WIRING_APPROACHES,
     IGRAPH_DEFS,
+    NAPISTU_GRAPH,
     NAPISTU_GRAPH_EDGES,
     NAPISTU_GRAPH_NODE_TYPES,
     NAPISTU_GRAPH_VERTICES,
     NAPISTU_METADATA_KEYS,
     NAPISTU_WEIGHTING_STRATEGIES,
+    WEIGHT_TRANSFORMATIONS,
     WEIGHTING_SPEC,
 )
 from napistu.network.ng_core import NapistuGraph
+from napistu.ontologies.constants import SPECIES_TYPES
 
 logger = logging.getLogger(__name__)
 
@@ -32,6 +35,8 @@ def test_graph():
     g.add_vertices(3, attributes={NAPISTU_GRAPH_VERTICES.NAME: ["A", "B", "C"]})
     g.add_edges([(0, 1), (1, 2)])
     g.es[SBML_DFS.R_ID] = ["R1", "R2"]
+    g.es[NAPISTU_GRAPH_EDGES.FROM] = ["A", "B"]
+    g.es[NAPISTU_GRAPH_EDGES.TO] = ["B", "C"]
     return NapistuGraph.from_igraph(g)
 
 
@@ -65,6 +70,165 @@ def mixed_node_types_graph():
     return g
 
 
+def test_remove_vertex_attributes():
+    """Test removing vertex attributes from a graph."""
+    g = ig.Graph()
+    g.add_vertices(
+        3,
+        attributes={
+            NAPISTU_GRAPH_VERTICES.NAME: ["A", "B", "C"],
+            "foo": ["protein", "metabolite", "protein"],
+            "bar": [1.0, 2.0, 3.0],
+        },
+    )
+    g.add_edges([(0, 1), (1, 2)])
+
+    napistu_graph = NapistuGraph.from_igraph(g)
+    napistu_graph.remove_attributes(NAPISTU_GRAPH.VERTICES, ["foo", "bar"])
+
+    assert "foo" not in napistu_graph.vs.attributes()
+    assert "bar" not in napistu_graph.vs.attributes()
+    assert NAPISTU_GRAPH_VERTICES.NAME in napistu_graph.vs.attributes()
+
+
+def test_remove_edge_attributes():
+    """Test removing edge attributes from a graph."""
+    g = ig.Graph()
+    g.add_vertices(3, attributes={NAPISTU_GRAPH_VERTICES.NAME: ["A", "B", "C"]})
+    g.add_edges([(0, 1), (1, 2)])
+    g.es["foo"] = [1.0, 2.0]
+    g.es["bar"] = [0.8, 0.9]
+
+    napistu_graph = NapistuGraph.from_igraph(g)
+    napistu_graph.remove_attributes(NAPISTU_GRAPH.EDGES, ["foo", "bar"])
+
+    assert "foo" not in napistu_graph.es.attributes()
+    assert "bar" not in napistu_graph.es.attributes()
+
+
+@pytest.mark.parametrize(
+    "attribute_type,entity_type,data_method,transform_method",
+    [
+        (
+            NAPISTU_GRAPH.VERTICES,
+            SBML_DFS.SPECIES,
+            "add_vertex_data",
+            "transform_vertices",
+        ),
+        (NAPISTU_GRAPH.EDGES, SBML_DFS.REACTIONS, "add_edge_data", "transform_edges"),
+    ],
+)
+def test_remove_attributes_metadata_cleanup(
+    attribute_type, entity_type, data_method, transform_method
+):
+    """Test that removing attributes cleans up metadata for both vertices and edges."""
+    # Create graph
+    g = ig.Graph()
+    g.add_vertices(
+        3,
+        attributes={
+            NAPISTU_GRAPH_VERTICES.NAME: ["A", "B", "C"],
+            SBML_DFS.S_ID: ["A", "B", "C"],  # Required for vertex data merging
+        },
+    )
+    g.add_edges([(0, 1), (1, 2)])
+    g.es[NAPISTU_GRAPH_EDGES.FROM] = ["A", "B"]  # Required for edge data merging
+    g.es[NAPISTU_GRAPH_EDGES.TO] = ["B", "C"]  # Required for edge data merging
+
+    napistu_graph = NapistuGraph.from_igraph(g)
+
+    # Create side-loaded data based on attribute type
+    if attribute_type == NAPISTU_GRAPH.VERTICES:
+        side_loaded_data = {
+            "test_table": pd.DataFrame(
+                {
+                    SBML_DFS.S_ID: ["A", "B", "C"],
+                    "foo": [1.0, 2.0, 3.0],
+                    "bar": [10, 20, 30],
+                }
+            ).set_index(SBML_DFS.S_ID)
+        }
+    else:  # edges
+        side_loaded_data = {
+            "test_table": pd.DataFrame(
+                {
+                    NAPISTU_GRAPH_EDGES.FROM: ["A", "B"],
+                    NAPISTU_GRAPH_EDGES.TO: ["B", "C"],
+                    "foo": [1.0, 2.0],
+                    "bar": [0.8, 0.9],
+                }
+            ).set_index([NAPISTU_GRAPH_EDGES.FROM, NAPISTU_GRAPH_EDGES.TO])
+        }
+
+    # Set up graph attributes for transformation
+    graph_attrs = {
+        entity_type: {
+            "foo": {
+                WEIGHTING_SPEC.TABLE: "test_table",
+                WEIGHTING_SPEC.VARIABLE: "foo",
+                WEIGHTING_SPEC.TRANSFORMATION: WEIGHT_TRANSFORMATIONS.STRING,
+            },
+            "bar": {
+                WEIGHTING_SPEC.TABLE: "test_table",
+                WEIGHTING_SPEC.VARIABLE: "bar",
+                WEIGHTING_SPEC.TRANSFORMATION: WEIGHT_TRANSFORMATIONS.IDENTITY,
+            },
+        }
+    }
+    napistu_graph.set_graph_attrs(graph_attrs)
+
+    # Add data using the appropriate method
+    getattr(napistu_graph, data_method)(side_loaded_attributes=side_loaded_data)
+
+    # Apply transformations to create metadata trail
+    getattr(napistu_graph, transform_method)(keep_raw_attributes=True)
+
+    # Verify metadata exists
+    assert NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES in napistu_graph._metadata
+    assert entity_type in napistu_graph._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES]
+    assert (
+        "foo"
+        in napistu_graph._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES][entity_type]
+    )
+    assert (
+        "bar"
+        in napistu_graph._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES][entity_type]
+    )
+    assert NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED in napistu_graph._metadata
+    assert (
+        entity_type
+        in napistu_graph._metadata[NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED]
+    )
+
+    # Remove one attribute
+    napistu_graph.remove_attributes(attribute_type, ["foo"])
+
+    # Verify foo metadata was cleaned up
+    assert (
+        "foo"
+        not in napistu_graph._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES][
+            entity_type
+        ]
+    )
+    assert (
+        "foo"
+        not in napistu_graph._metadata[NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED][
+            entity_type
+        ]
+    )
+    # But bar should still be there
+    assert (
+        "bar"
+        in napistu_graph._metadata[NAPISTU_METADATA_KEYS.RAW_ATTRIBUTES][entity_type]
+    )
+    assert (
+        "bar"
+        in napistu_graph._metadata[NAPISTU_METADATA_KEYS.TRANSFORMATIONS_APPLIED][
+            entity_type
+        ]
+    )
+
+
 def test_remove_isolated_vertices():
     """Test removing isolated vertices from a graph."""
 
@@ -171,6 +335,100 @@ def test_to_pandas_dfs():
     )
 
 
+def test_vertex_dataframe_ordering_preservation():
+    """Test that get_vertex_dataframe preserves the original vertex ordering."""
+    # Create a graph with vertices in a specific order
+    g = NapistuGraph(directed=True)
+
+    # Add vertices with specific names in a particular order
+    vertex_names = ["C", "A", "B", "D"]
+    for name in vertex_names:
+        g.add_vertex(
+            name=name, node_type=SBML_DFS.SPECIES, species_type=SPECIES_TYPES.METABOLITE
+        )
+
+    # Add some edges
+    g.add_edge("A", "B")
+    g.add_edge("B", "C")
+    g.add_edge("C", "D")
+
+    # Get vertex dataframe
+    vertex_df = g.get_vertex_dataframe()
+
+    # Verify ordering is preserved (should match igraph's internal order)
+    assert len(vertex_df) == 4
+    assert list(vertex_df[NAPISTU_GRAPH_VERTICES.NAME]) == vertex_names
+
+    # The index should be named 'index' and contain sequential values
+    assert vertex_df.index.name == IGRAPH_DEFS.INDEX
+    assert list(vertex_df.index) == list(range(4))  # 0, 1, 2, 3
+
+    # Verify that the index matches the DataFrame row position
+    for i, row in vertex_df.iterrows():
+        assert i == row.name  # row.name is the index value
+
+
+def test_edge_dataframe_ordering_preservation():
+    """Test that get_edge_dataframe preserves the original edge ordering."""
+    # Create a graph with edges in a specific order
+    g = NapistuGraph(directed=True)
+
+    # Add vertices
+    g.add_vertex(name="A", node_type=SBML_DFS.SPECIES)
+    g.add_vertex(name="B", node_type=SBML_DFS.SPECIES)
+    g.add_vertex(name="C", node_type=SBML_DFS.SPECIES)
+    g.add_vertex(name="D", node_type=SBML_DFS.SPECIES)
+
+    # Add edges in a specific order
+    edge_pairs = [("A", "B"), ("B", "C"), ("C", "D"), ("A", "D")]
+    for source, target in edge_pairs:
+        g.add_edge(source, target, weight=1.0)
+
+    # Get edge dataframe
+    edge_df = g.get_edge_dataframe()
+
+    # Verify ordering is preserved
+    assert len(edge_df) == 4
+    assert list(edge_df[IGRAPH_DEFS.SOURCE]) == [0, 1, 2, 0]  # A=0, B=1, C=2, D=3
+    assert list(edge_df[IGRAPH_DEFS.TARGET]) == [1, 2, 3, 3]  # B=1, C=2, D=3, D=3
+
+    # Verify that edge attributes are in the same order
+    assert list(edge_df[NAPISTU_GRAPH_EDGES.WEIGHT]) == [1.0, 1.0, 1.0, 1.0]
+
+
+def test_to_pandas_dfs_ordering_consistency():
+    """Test that to_pandas_dfs maintains consistent ordering between vertices and edges."""
+    # Create a graph with known structure
+    g = NapistuGraph(directed=True)
+
+    # Add vertices in specific order
+    vertex_names = ["X", "Y", "Z"]
+    for name in vertex_names:
+        g.add_vertex(name=name, node_type=SBML_DFS.SPECIES, value=len(name))
+
+    # Add edges
+    g.add_edge("X", "Y", edge_type="interaction")
+    g.add_edge("Y", "Z", edge_type="regulation")
+
+    # Get both dataframes
+    vertex_df, edge_df = g.to_pandas_dfs()
+
+    # Verify vertex ordering
+    assert list(vertex_df[NAPISTU_GRAPH_VERTICES.NAME]) == vertex_names
+    assert list(vertex_df[IGRAPH_DEFS.INDEX]) == list(range(3))
+
+    # Verify edge ordering and that source/target indices are valid
+    assert len(edge_df) == 2
+    assert list(edge_df[IGRAPH_DEFS.SOURCE]) == [0, 1]  # X=0, Y=1
+    assert list(edge_df[IGRAPH_DEFS.TARGET]) == [1, 2]  # Y=1, Z=2
+    assert list(edge_df["edge_type"]) == ["interaction", "regulation"]
+
+    # Verify all edge indices are valid vertex indices
+    max_vertex_idx = vertex_df[IGRAPH_DEFS.INDEX].max()
+    assert edge_df[IGRAPH_DEFS.SOURCE].max() <= max_vertex_idx
+    assert edge_df[IGRAPH_DEFS.TARGET].max() <= max_vertex_idx
+
+
 def test_set_and_get_graph_attrs(test_graph):
     """Test setting and getting graph attributes."""
     attrs = {
@@ -303,7 +561,7 @@ def test_graph_attrs_extend_and_overwrite_protection(test_graph):
 def test_add_edge_data_basic_functionality(test_graph, minimal_valid_sbml_dfs):
     """Test basic add_edge_data functionality with mock reaction data."""
     # Update the test graph to have the correct r_ids that match the SBML data
-    test_graph.es["r_id"] = [
+    test_graph.es[SBML_DFS.R_ID] = [
         "R00001",
         "R00001",
     ]  # Both edges should map to the same reaction
@@ -349,7 +607,7 @@ def test_add_edge_data_basic_functionality(test_graph, minimal_valid_sbml_dfs):
 def test_add_edge_data_mode_and_overwrite(test_graph, minimal_valid_sbml_dfs):
     """Test mode and overwrite behavior for add_edge_data."""
     # Update the test graph to have the correct r_ids that match the SBML data
-    test_graph.es["r_id"] = [
+    test_graph.es[SBML_DFS.R_ID] = [
         "R00001",
         "R00001",
     ]  # Both edges should map to the same reaction
@@ -1540,3 +1798,152 @@ def test_add_edge_data_side_loaded_only(napistu_graph):
     assert any(
         source is not None and not pd.isna(source) for source in source_dbs
     ), "No valid source_db values found"
+
+
+def test_show_summary(napistu_graph):
+    """Test that show_summary method runs without errors."""
+
+    # Call show_summary - should not raise any exceptions
+    napistu_graph.show_summary()
+
+    # If we get here, the method ran successfully
+    # We can also verify that get_summary works and returns expected structure
+    summary_stats = napistu_graph.get_summary()
+
+    # Print the actual summary stats for inspection
+    print("\n=== Summary Stats ===")
+    print(f"n_vertices: {summary_stats['n_vertices']}")
+    print(f"n_edges: {summary_stats['n_edges']}")
+    print(f"vertex_node_type_dict: {summary_stats['vertex_node_type_dict']}")
+    print(f"vertex_species_type_dict: {summary_stats['vertex_species_type_dict']}")
+    print(f"sbo_name_counts_dict: {summary_stats['sbo_name_counts_dict']}")
+    print(f"vertex_attributes: {summary_stats['vertex_attributes']}")
+    print(f"edge_attributes: {summary_stats['edge_attributes']}")
+    print("===================\n")
+
+    # Verify the summary has the expected keys
+    expected_keys = [
+        "n_vertices",
+        "vertex_node_type_dict",
+        "vertex_species_type_dict",
+        "vertex_attributes",
+        "n_edges",
+        "sbo_name_counts_dict",
+        "edge_attributes",
+    ]
+
+    for key in expected_keys:
+        assert key in summary_stats, f"Missing key: {key}"
+
+    # Test vertices: type and count
+    assert isinstance(summary_stats["n_vertices"], int)
+    assert (
+        summary_stats["n_vertices"] == 30
+    ), f"Expected 30 vertices, got {summary_stats['n_vertices']}"
+
+    # Test edges: type and count
+    assert isinstance(summary_stats["n_edges"], int)
+    assert (
+        summary_stats["n_edges"] == 32
+    ), f"Expected 32 edges, got {summary_stats['n_edges']}"
+
+    # Test vertex node type breakdown: type and content
+    assert isinstance(summary_stats["vertex_node_type_dict"], dict)
+    expected_node_types = {
+        NAPISTU_GRAPH_NODE_TYPES.SPECIES: 23,
+        NAPISTU_GRAPH_NODE_TYPES.REACTION: 7,
+    }
+    assert (
+        summary_stats["vertex_node_type_dict"] == expected_node_types
+    ), f"Expected {expected_node_types}, got {summary_stats['vertex_node_type_dict']}"
+
+    # Test vertex species type breakdown: type and content
+    assert isinstance(summary_stats["vertex_species_type_dict"], dict)
+    expected_species_types = {"metabolite": 13, "complex": 9, "protein": 1}
+    assert (
+        summary_stats["vertex_species_type_dict"] == expected_species_types
+    ), f"Expected {expected_species_types}, got {summary_stats['vertex_species_type_dict']}"
+
+    # Test SBO term breakdown: type and content
+    assert isinstance(summary_stats["sbo_name_counts_dict"], dict)
+    expected_sbo_terms = {
+        SBOTERM_NAMES.PRODUCT: 13,
+        SBOTERM_NAMES.REACTANT: 13,
+        SBOTERM_NAMES.CATALYST: 6,
+    }
+    assert (
+        summary_stats["sbo_name_counts_dict"] == expected_sbo_terms
+    ), f"Expected {expected_sbo_terms}, got {summary_stats['sbo_name_counts_dict']}"
+
+    # Test vertex attributes: type and content
+    assert isinstance(summary_stats["vertex_attributes"], list)
+    expected_vertex_attrs = {
+        NAPISTU_GRAPH_VERTICES.NAME,
+        NAPISTU_GRAPH_VERTICES.NODE_NAME,
+        NAPISTU_GRAPH_VERTICES.NODE_TYPE,
+        NAPISTU_GRAPH_VERTICES.SPECIES_TYPE,
+        SBML_DFS.S_ID,
+        SBML_DFS.C_ID,
+    }
+    actual_vertex_attrs = set(summary_stats["vertex_attributes"])
+    assert (
+        actual_vertex_attrs == expected_vertex_attrs
+    ), f"Expected {expected_vertex_attrs}, got {actual_vertex_attrs}"
+
+    # Test edge attributes: type and content
+    assert isinstance(summary_stats["edge_attributes"], list)
+    expected_edge_attrs = {
+        NAPISTU_GRAPH_EDGES.FROM,
+        NAPISTU_GRAPH_EDGES.TO,
+        NAPISTU_GRAPH_EDGES.STOICHIOMETRY,
+        NAPISTU_GRAPH_EDGES.SBO_TERM,
+        SBML_DFS.R_ID,
+        NAPISTU_GRAPH_EDGES.SPECIES_TYPE,
+        SBML_DFS.R_ISREVERSIBLE,
+        NAPISTU_GRAPH_EDGES.DIRECTION,
+        NAPISTU_GRAPH_EDGES.SC_DEGREE,
+        NAPISTU_GRAPH_EDGES.SC_CHILDREN,
+        NAPISTU_GRAPH_EDGES.SC_PARENTS,
+        "topo_weights",
+        "upstream_topo_weights",
+        NAPISTU_GRAPH_EDGES.WEIGHT,
+        NAPISTU_GRAPH_EDGES.UPSTREAM_WEIGHT,
+    }
+    actual_edge_attrs = set(summary_stats["edge_attributes"])
+    assert (
+        actual_edge_attrs == expected_edge_attrs
+    ), f"Expected {expected_edge_attrs}, got {actual_edge_attrs}"
+
+
+def test_get_vertex_series(test_graph):
+    """Test get_vertex_series method."""
+    # Test getting vertex names
+    vertex_names = test_graph.get_vertex_series(NAPISTU_GRAPH_VERTICES.NAME)
+    expected_names = pd.Series(
+        ["A", "B", "C"], index=["A", "B", "C"], name=NAPISTU_GRAPH_VERTICES.NAME
+    )
+    expected_names.index.name = NAPISTU_GRAPH_VERTICES.NAME
+    pd.testing.assert_series_equal(vertex_names, expected_names)
+
+    # Test getting non-existent attribute
+    with pytest.raises(KeyError, match="Vertex attribute 'nonexistent' not found"):
+        test_graph.get_vertex_series("nonexistent")
+
+
+def test_get_edge_series(test_graph):
+    """Test get_edge_series method."""
+    # Test getting edge R_ID attribute
+    edge_r_ids = test_graph.get_edge_series(NAPISTU_GRAPH_EDGES.R_ID)
+    expected_r_ids = pd.Series(
+        ["R1", "R2"],
+        index=pd.MultiIndex.from_tuples(
+            [("A", "B"), ("B", "C")],
+            names=[NAPISTU_GRAPH_EDGES.FROM, NAPISTU_GRAPH_EDGES.TO],
+        ),
+        name=NAPISTU_GRAPH_EDGES.R_ID,
+    )
+    pd.testing.assert_series_equal(edge_r_ids, expected_r_ids)
+
+    # Test getting non-existent attribute
+    with pytest.raises(KeyError, match="Edge attribute 'nonexistent' not found"):
+        test_graph.get_edge_series("nonexistent")
